from datetime import date as dtdate, datetime
from pathlib import Path
from typing import Optional, Union

from opa_tools.config import BENCHMARK_DIR

DAY_DIGITS = 3


def _validate_exists(path: Path):
    """
    Raises FileNotFoundError if the given path doesn't exist.
    """
    if not path.exists():
        raise FileNotFoundError(f"Directory {path} not found.")


def get_benchmarking_path(
    hostname: Optional[str] = None,
    date: Optional[Union[str, dtdate, datetime]] = None,
    num: Optional[int] = None,
    no_exist_ok: bool = True,
) -> Path:
    """
    Returns a benchmark path based on the given date and id.
    If num is None, returns the day-level directory.
    """
    if date is None:
        date = datetime.now()
    if isinstance(date, (dtdate, datetime)):
        date = date.strftime(r"%Y%m%d")

    day_benchmark_path = BENCHMARK_DIR / date

    if num is None:
        if not no_exist_ok:
            _validate_exists(day_benchmark_path)
        return day_benchmark_path

    benchmark_path = day_benchmark_path / f"{num:0{DAY_DIGITS}d}"
    if not no_exist_ok:
        _validate_exists(benchmark_path)
    return benchmark_path
